@import url('https://fonts.googleapis.com/css2?family=Roboto+Mono:wght@500;700&display=swap');

/* https://www.paulirish.com/2012/box-sizing-border-box-ftw/ */
html {
    box-sizing: border-box;
}

*, *:before, *:after {
    box-sizing: inherit;
}


* {
    font-family: 'Roboto Mono', monospace;
}

body {
    background-color: #1d2021;
}

p,b,h1,h2,tr,td {
    color: #ebdbb2;
}

h1 {
    border-bottom: 1px #928374 solid;
    padding-bottom: 5px;
}

b {
    font-family: 'Roboto Mono Bold', monospace;
}

li {
    list-style: none;
    margin-left: 30px;
}


p,b {
    margin-left: 5px;
    margin-right: 5px;
}


button {
    color: #ebdbb2;
    background-color: #1d2021;
    transition-duration: 0.4s;
    border: none;
    border-radius: 4px;
    padding: 5px 10px;
}

button:hover {
    background-color: #458588;  /* TODO: color customization from app */
}

input {
    background-color: #1d2021;
    color: #ebdbb2;
    width: 100%;
    padding: 10px 20px;
    margin: 10px 0;
    border: 1px solid #928374;
    border-radius: 4px;
    outline: none;
    width: 100%
}

input:focus {
    border: 1px solid #fbf1c7;
}

table {
    width: 100%;
    padding: 10px;
    text-align: left;
    border-collapse: collapse;
}

th {
    color: #83a598;
}

th,td {
    padding: 10px 5px;
    border-bottom: 1px solid #928374;
}

tr:hover {
    background-color: #282828;
}


#app {
    width: 1000px;
    margin-left: auto;
    margin-right: auto;
}


#menu {
    margin-bottom: 20px;

    b {
        color: #fbf1c7;
    }
}